/*
	Plasmosis, real time plasma style effect.
		
    Much of this material is experimental in nature, and may have
    flaws that can cause failure of your or others' computer and
    network systems. The author offers no support for this material,
    except for limited assistance to formal participants in  recognized
    field test programs.
    
    The author makes no warrantee as to the correctness, usefulness, or
    fitness for any particular purpose of any of the material in this file.
    The author further disclaims all responsibility for damage to your
    computers, networks, or data or those of third parties, as well as
    all other damages, including incidental or consequential damages,
    arising from your or others' use of this material. By using or
    redistributing any of this material, you assume all responsibility
    for the results of your use or redistribution.
	
	David C. Finch  19th August 1997
	<dave-f@dircon.co.uk>
	http://www.users.dircon.co.uk/~dave-f
*/

#include "Plasmosis.h"


void main(void);
void Exit(void);
void Plasmosisizolation(void);


void main(void)
	{
		short			depth,hd;
		GDHandle		dev;
		CWindowPtr		window;
		PaletteHandle	pal;
		
		
		//	Fairly standard toolbox init.
		MaxApplZone();
		MoreMasters();
		InitGraf(&qd.thePort);
		InitFonts();
		InitWindows();
		InitMenus();
		TEInit();
		InitDialogs(0);
		InitCursor();
		
		//	Randomize the randomizer random seed.
		qd.randSeed = TickCount();
		
		//	Get the pixel depth of the current screen.
		dev = 0;
		dev = GetGDevice();
		if(dev == 0) Exit();
		depth = (*(*dev)->gdPMap)->pixelSize;
		
		//	If we are in 256 colour mode then carry on.
		if(depth != 8)
			{
				//	Otherwise we must check that we have 256 colours available.
				hd = HasDepth(dev,8,0,0);
				//	If we don't have 256 colours available we inform the user and quit.
				if(hd == 0)
					{
						Alert(128,0);
						Exit();
					}
				
				//	Ask the user if we can switch colours modes.
				//	If the user says "Absolutely no way, never !  Now leave !" we quit.
				if(Alert(129,0) == 2) Exit();
				
				//	256 colours available, user say okay, we change.
				SetDepth(dev,8,0,0);
			}
		
		//	Moving cursors make a mess of direct screen addressing so we hide it.
		HideCursor();
		
		//	Get a new colour window.
		window = (CWindowPtr)NewCWindow(0,&qdb,"\p",true,plainDBox,(WindowPtr)-1L,false,0);
		if(!window) Exit();
		SetPort((WindowPtr)window);
		
		//	Load the "rainbow" palette.
		pal = GetNewPalette(128);
		if(!pal) Exit();
		HLockHi((Handle)pal);
		SetPalette((WindowPtr)window,pal,true);
		
		//	jump into the main routine. (Excuse the name).
		Plasmosisizolation();
		
		//	If we weren't already in 256 colour mode switch back to the original mode.
		if(depth != 8) SetDepth(dev,depth,0,0);
		
		//	Now leave.
		Exit();
	}


void Exit(void)
	{
		//	As we used a "while(!Button())" in the main loop we should flush the event queue.
		FlushEvents(everyEvent,0);
		
		//	Not actually required as we're quitting anyway but good practice.
		ShowCursor();
		
		//	Bye !
		ExitToShell();
	}


void Plasmosisizolation(void)
	{
		long		x,y;
		long		mx,my;
		long		c,d;
		long		ix,iy;
		long		*sqrlut;
		long		col,whProd,ef,iter,ticks;
		long		px[3],py[3];
		double		pxv[3],pyv[3];
		double		pxvv[3],pyvv[3];
		double		cf;
		
		
		//	Set up a few constants used in the calculations, etc.
		//	'ixMax' and 'iyMax'.  One must be even, one must be odd.  Higher values = faster calcs.
		//	'maxVel', anything over about 8 gets really messy.
		//	'wew' again, the higher, the messier.  Anything will go except zero (not checked).
		//	'maxIter' don't go lower that around 100 as it gets annoying.
		#define		maxVel			4		//	Maximum point velocity.
		#define		ixMax			3		//	X-axis interpolation factor.
		#define		iyMax			4		//	Y-axis interpolation factor.
		#define		wew				3		//	Wierd effect wierdness.
		#define		maxIter			500		//	Maximum loops before re-initializing values.
		
		//	Clear the screen. (No need to hide menu bar, control strip, etc.)
		for(y = 0; y < qdb.bottom; y ++) for(x = 0; x < qdb.right; x ++) DSA(x,y) = 255;
		
		//	Calculate the product of screen width * height.
		whProd = qdb.right * qdb.bottom;
		
		//	Calculate the "wierd effect" factor. (Couldn't think of anything else to call it :)
		ef = whProd / wew;
		
		//	Create the square root look-up table.
		sqrlut = (long*)NewPtr(whProd * 4);
		if(!sqrlut)
			{
				ShowCursor();
				Alert(130,0);
				return;
			}
		for(c = 0; c < whProd; c ++) sqrlut[c] = sqrt(c);
		
		//	Calculate the colour division factor.
		cf = 256.0 / sqrt((qdb.right * qdb.right) + (qdb.bottom * qdb.bottom));
		cf /= 4;
		
		//	Initialize the interpolation counters.
		ix = 0;
		iy = 0;
		
		//	Initialize the iteration counter.
		iter = 0;
		
		//	Initialize the 3 random points and their respective velocities.
start:	for(c = 0; c < 3; c ++)
			{
				px[c] = ((Random() + 32768) * qdb.right) / 65535;
				py[c] = ((Random() + 32768) * qdb.bottom) / 65535;
				
				pxv[c] = (Random() * maxVel) / 32767;
				pyv[c] = (Random() * maxVel) / 32767;
			}
		pxvv[0] = 0.03;
		pxvv[1] = -0.04;
		pxvv[2] = 0.05;
		pyvv[0] = 0.01;
		pyvv[1] = 0.06;
		pyvv[2] = -0.04;
		
		//	Now into the main loop.
		while(!Button())
			{
				//	Get the current system tick count, used for delay purposes at the end of the loop.
				ticks = TickCount();
				
				//	Update the 3 point velocities and the point positions respectively.
				for(c = 0; c < 3; c ++)
					{
						pxv[c] += pxvv[c];
						pyv[c] += pyvv[c];
						
						if(pxv[c] > maxVel || pxv[c] < -maxVel)
							{
								pxvv[c] = -pxvv[c];
								pxv[c] += pxvv[c];
							}
						if(pyv[c] > maxVel || pyv[c] < -maxVel)
							{
								pyvv[c] = -pyvv[c];
								pyv[c] += pyvv[c];
							}
							
						px[c] += pxv[c];
						py[c] += pyv[c];
					}
				
				//	Update the X & Y iteration values and reset if needed.
				ix ++;
				if(ix > ixMax) ix = 0;
				
				iy ++;
				if(iy > iyMax) iy = 0;
				
				//	Now do the business.
				//	I'm deliberately leaving this uncommented, uses Pythagoras theorem.
				//	See "Plasmosis.h" for the DSA(x,y) definition.
				for(y = iy; y < qdb.bottom; y += iyMax + 1)
					{
						for(x = ix; x < qdb.right; x += ixMax + 1)
							{
								col = 0;
								
								for(c = 0; c < 3; c ++)
									{
										mx =  x - px[c];
										my =  y - py[c];
										
										mx *= mx;
										my *= my;
										
										d = (mx + my) % ef;
										
										col += sqrlut[d];
									}
								
								col /= 3;
								col *= cf;
								
								DSA(x,y) = col;
							}
					}
				
				//	Have we seen enough frames using this set of values ?  If so, reset.
				iter ++;
				if(iter > maxIter)
					{
						iter = 0;
						goto start;
					}
				
				//	If this frame hasn't taken one full 'tick count' then wait here until one has passed.
				if(ticks == TickCount()) Delay(1,&ticks);
			}
	}


